import enum
import uuid

from flask_sqlalchemy import SQLAlchemy
from sqlalchemy import Enum
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.sql import func

db = SQLAlchemy()


class StatusEnum(enum.Enum):
    COMPLETED = "COMPLETED"
    ONPROGRESS = "ONPROGRESS"


class Document(db.Model):
    id = db.Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = db.Column(db.String(32), nullable=False)
    URL = db.Column(db.String(256), nullable=True)
    status = db.Column(Enum(StatusEnum), nullable=False, default=StatusEnum.ONPROGRESS)
    user_id = db.Column(UUID(as_uuid=True), nullable=False)
    created_at = db.Column(db.DateTime(timezone=True), server_default=func.now())

    def __repr__(self):
        return f"<Document {self.name}>"


class Template(db.Model):
    id = db.Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    name = db.Column(db.String(32), nullable=False)
    URL = db.Column(db.String(256), nullable=False)
    user_id = db.Column(UUID(as_uuid=True), nullable=False)

    def __repr__(self):
        return f"<Template {self.id}>"
